@echo off
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: CopySTUModeDefaults.cmd
:: 	Copies a custom default file with defaults to put the DTU-1031
:: 	into "STU mode" to system location where it will be found by the 
:: 	Wacom tablet driver and loaded during driver initialization.
::
:: Params:
:: 	-q = execute in quiet mode
::
:: Notes:
:: 	The result of executing this script is to copy a user custom 
:: 	defaults file to a system location where it can be found and merged
:: 	with existing user defaults by the wacom tablet installer.
:: 	The custom settings copied by this script will disable tablet pen
:: 	and button input except for those applications that have either
:: 	programmed to the Wacom pen and button APIs, or applications that
:: 	have been added through the Wacom tablet control panel.
::
:: 	This functionality is supported by tablet driver 6.3.5 and higher.
::
:: Copyright Wacom Technology, Inc. 2013
:: All rights reserved.
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

set QUIETOPT=%1

:: This script will set environment params for the file path we need.
call SetWacomUserParams.cmd

if "%USERDEFAULTSFILE%"=="" goto ERRPARAMS
if "%USERDEFAULTSFILETEMPDIR%"=="" goto ERRPARAMS

if NOT EXIST .\%USERDEFAULTSFILE% goto FILENOTFOUND

set XCOPYCMD=XCOPY %USERDEFAULTSFILE% %USERDEFAULTSFILETEMPDIR%
set XCOPYFLAGS=/QYI ^> NUL

if "%QUIETOPT%"=="%QUIETFLAG%" GOTO DOCOPY
echo.
echo Copying %USERDEFAULTSFILE% to temp location...
set XCOPYFLAGS=/FYI

:DOCOPY
%XCOPYCMD% %XCOPYFLAGS%

goto :EOF

:ERRPARAMS
echo.
echo *********************************************
echo *** ERROR: Did you run SetWacomuserParams?
echo *********************************************
echo.
pause
goto :EOF

:FILENOTFOUND
echo.
echo ********************************************************
echo *** ERROR: .\%USERDEFAULTSFILE%" not found.  
echo ********************************************************
echo.
pause
goto :EOF
