@echo off
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: RestartDriver.bat
:: 	Restart the tablet driver with any custom settings.
::
:: USAGE:
::	RestartDriver [-q] [exedir]
::
:: PARAMS:
:: 	-q = execute in quiet mode
::      exedir = optional directory for setup.exe
::
:: Notes:
:: 	This script removes tablet preferences, merges custom tablet 
:: 	defaults (if any), and restarts the tablet driver.
::
:: 	The result of executing this script will depend on whether it 
:: 	finds any custom tablet defaults or not.  If custom defaults are
:: 	found, then they will be read in affecting tablet usage.  In
:: 	either case (customizations or not), the tablet preferences will
:: 	be removed, and the driver will be restarted.
::
:: 	This functionality is supported by tablet driver 6.3.5 and higher.
::
:: Copyright Wacom Technology, Inc. 2013
:: All rights reserved.
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

set QUIETOPT=

:: If user did not specify a place to look for tablet install files,
:: then use the default location in the installer folder.
set EXEDIR=..

:: -----------------------------------------------------------------------
:: This script will set environment params for the file path we need.
call SetWacomUserParams.cmd

:: -----------------------------------------------------------------------
:: Make sure the above script actually set some params.
if "%USERDEFAULTSFILETEMP%"=="" goto ERRPARAMS

:: -----------------------------------------------------------------------
:: Set script options.
::
:PARSEOPTS
    if "%1"=="%QUIETFLAG%" goto OPTQ
    if "%1"=="%EXEDIRFLAG%" goto OPTD
    goto CONTINUE

    :OPTQ
    set QUIETOPT=%1
    shift
    goto PARSEOPTS

    :OPTD
    shift
    set EXEDIR=%1
    shift
    goto PARSEOPTS

:CONTINUE

:: -----------------------------------------------------------------------
:: Make sure that we can find the tools we need.
set PREFUTILEXE=%EXEDIR%\prefutil.exe
set SETUPEXE=%EXEDIR%\setup.exe

if NOT EXIST %SETUPEXE% goto ERRSETUP
if NOT EXIST %PREFUTILEXE% goto ERRPREFUTIL

if "%QUIETOPT%"=="%QUIETFLAG%" GOTO DOSETUP

echo.
echo *************************************************************************************
echo WARNING!
echo.
echo     This script WILL remove existing user preferences and reinstall the driver. 
echo     Any existing user default settings will be preserved.
echo.
echo     This installer will be used: %SETUPEXE%
echo.
echo Proceed?
echo *************************************************************************************
echo.
pause

:: -----------------------------------------------------------------------
:: Run installer (setup.exe) in silent mode, which will merge 
:: %USERDEFAULTSFILETEMP% (if found) into file of the same name in 
:: <drive>\program files\Tablet\Wacom, and then will start the driver.  
:: The driver may, or may not prompt to reboot.
::
echo.
echo Merging custom defaults and reinstalling (Hang on, this may take several minutes) ...
echo %SETUPEXE% ^/silent

:DOSETUP
"%SETUPEXE%" /silent

if "%QUIETOPT%"=="%QUIETFLAG%" GOTO DOREMOVEPREFS

:: -----------------------------------------------------------------------
:: Run prefutil, which will remove all prefs.
::
echo.
echo Removing prefs ...
echo %PREFUTILEXE% ^/remove-all

:DOREMOVEPREFS
"%PREFUTILEXE%" /remove-all

if "%QUIETOPT%"=="%QUIETFLAG%" GOTO :EOF

echo.
echo ****************************************************
echo.
echo       Do a system reboot as soon as possible!
echo.
echo ****************************************************
echo.
pause

GOTO :EOF

:ERRPARAMS
echo.
echo *********************************************
echo *** ERROR: USERDEFAULTSFILETEMP not found.  
echo ***
echo ***        Did you run SetWacomuserParams?
echo *********************************************
echo.
pause
goto :EOF

:ERRSETUP
echo.
echo ****************************************************************************************
echo *** ERROR: %SETUPEXE% not found.  
echo ***
echo ***        Cannot find installer to run. Check your configuration.
echo ****************************************************************************************
echo.
pause
goto :EOF

:ERRSETUP
echo.
echo ****************************************************************************************
echo *** ERROR: %PREFUTILEXE% not found.  
echo ***
echo ***        Cannot find prefutil to run. Check your configuration.
echo ****************************************************************************************
echo.
pause
goto :EOF

